/*
 * Copyright Staffan Gimåker 2010.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PEEKABOT_FLOAT_PROP_BASE_HH_INCLUDED
#define PEEKABOT_FLOAT_PROP_BASE_HH_INCLUDED


#include "Prop.hh"
#include "PropVisitor.hh"

#include <limits>


namespace peekabot
{
    class FloatPropBase : public Prop
    {
    public:
        virtual float get_min_value() const
        {
            return -std::numeric_limits<float>::max();
        }

        virtual float get_max_value() const
        {
            return std::numeric_limits<float>::max();
        }

        virtual bool display_as_slider() const
        {
            return false;
        }

        virtual float display_factor() const
        {
            return 1.0f;
        }

        virtual void accept(PropVisitor &visitor)
        {
            visitor.visit(this);
        }
    };
}


#endif // PEEKABOT_FLOAT_PROP_BASE_HH_INCLUDED
